<?php
/*
 * File name: RoleController.php
 * Last modified: 2021.03.18 at 16:44:59
 * Author: SmarterVision - https://codecanyon.net/user/smartervision
 * Copyright (c) 2024
 */

namespace App\Http\Controllers;

use App\DataTables\RoleDataTable;
use App\Http\Requests\CreateRoleRequest;
use App\Http\Requests\UpdateRoleRequest;
use App\Repositories\RoleRepository;
use Flash;
use Illuminate\Http\RedirectResponse;
use Illuminate\View\View;
use Response;

class RoleController extends Controller
{
    /** @var  RoleRepository */
    private RoleRepository $roleRepository;

    public function __construct(RoleRepository $roleRepo)
    {
        parent::__construct();
        $this->roleRepository = $roleRepo;
    }

    /**
     * Display a listing of the Role.
     *
     * @param RoleDataTable $roleDataTable
     * @return mixed
     */
    public function index(RoleDataTable $roleDataTable):mixed
    {
        return $roleDataTable->render('settings.roles.index');
    }

    /**
     * Show the form for creating a new Role.
     *
     * @return View
     */
    public function create():View
    {
        return view('settings.roles.create');
    }

    /**
     * Store a newly created Role in storage.
     *
     * @param CreateRoleRequest $request
     *
     * @return RedirectResponse
     */
    public function store(CreateRoleRequest $request):RedirectResponse
    {
        if ((config('installer.demo_app'))) {
            Flash::warning('This is only demo app you can\'t change this section ');
            return redirect(route('roles.index'));
        }
        $input = $request->all();

        $role = $this->roleRepository->create($input);

        Flash::success('Role saved successfully.');

        return redirect(route('roles.index'));
    }

    /**
     * Display the specified Role.
     *
     * @param  int $id
     *
     * @return RedirectResponse|View
     */
    public function show(int $id):RedirectResponse|View
    {
        $role = $this->roleRepository->findWithoutFail($id);

        if (empty($role)) {
            Flash::error('Role not found');

            return redirect(route('roles.index'));
        }

        return view('settings.roles.show')->with('role', $role);
    }

    /**
     * Show the form for editing the specified Role.
     *
     * @param  int $id
     *
     * @return RedirectResponse|View
     */
    public function edit(int $id):RedirectResponse|View
    {
        $role = $this->roleRepository->findWithoutFail($id);

        if (empty($role)) {
            Flash::error('Role not found');

            return redirect(route('roles.index'));
        }

        return view('settings.roles.edit')->with('role', $role);
    }

    /**
     * Update the specified Role in storage.
     *
     * @param  int              $id
     * @param UpdateRoleRequest $request
     *
     * @return RedirectResponse
     */
    public function update(int $id, UpdateRoleRequest $request):RedirectResponse
    {
        if(env('APP_DEMO',false)) {
            Flash::warning('This is only demo app you can\'t change this section ');
            return redirect(route('roles.index'));
        }
        $role = $this->roleRepository->findWithoutFail($id);

        if (empty($role)) {
            Flash::error('Role not found');

            return redirect(route('roles.index'));
        }

        $role = $this->roleRepository->update($request->all(), $id);

        Flash::success('Role updated successfully.');

        return redirect(route('roles.index'));
    }

    /**
     * Remove the specified Role from storage.
     *
     * @param  int $id
     *
     * @return RedirectResponse
     */
    public function destroy(int $id):RedirectResponse
    {
        $role = $this->roleRepository->findWithoutFail($id);

        if (empty($role)) {
            Flash::error('Role not found');

            return redirect(route('roles.index'));
        }

        $this->roleRepository->delete($id);

        Flash::success('Role deleted successfully.');

        return redirect(route('roles.index'));
    }
}
